# Příklad: rodokmen
# objekty typu Person reprezentují osoby včetně informace o rodiči a dětech
# (pro zjednodušení předpokládáme jen jednoho rodiče v rodokmenu)


class Person:
    def __init__(self, name):
        self.name = name
        self.parent = None
        self.children = []

    def add_child(self, child):
        self.children.append(child)
        child.parent = self


# jsou osoby a, b sourozenci?
def siblings(a, b):
    return a.parent != None and a.parent == b.parent


# kdo je nejstarší předek zadané osoby v rodokmenu?
def oldest_ancestor(person):
    while person.parent != None:
        person = person.parent
    return person


# kolik (přímých i nepřímých) potomků má daná osoba?
def count_offspring(person):
    count = 0
    for child in person.children:
        count += 1 + count_offspring(child)
    return count


# textové vykresleni rodokmenu
def draw_family_tree(person, level=0):
    print(("    " * level) + person.name)
    for child in person.children:
        draw_family_tree(child, level + 1)


# pomocná funkce
def add_transitions(person):
    for child in person.children:
        print('"' + person.name + '" -> "' + child.name + '"')
        add_transitions(child)


# vykreslení rodokmenu ve formátu GraphViz
# výstup můžete zkusit použít například na www.webgraphviz.com
def create_family_tree_graph(person):
    print("digraph {")
    add_transitions(person)
    print("}")


# nějaké rodiny na vyzkoušení
abe = Person("Abraham Simpson")
homer = Person("Homer Simpson")
abe.add_child(homer)
homer.add_child(Person("Bart Simpson"))
homer.add_child(Person("Lisa Simpson"))
homer.add_child(Person("Maggie Simpson"))
bart, lisa, maggie = homer.children

queen = Person("Queen Elisabeth II")
charles = Person("Prince Charles")
anne = Person("Princess Anne")
andrew = Person("Prince Andrew")
edward = Person("Prince Edward")
william = Person("Prince William")
henry = Person("Prince Henry")
george = Person("Prince George")
charlotte = Person("Princess Charlotte")
beatrice = Person("Princess Beatrice")
eugenie = Person("Princess Eugenie")
louise = Person("Lady Louise Windsor")
james = Person("James, Viscount Severn")
peter = Person("Peter Phillips")
savannah = Person("Savannah Phillips")
isla = Person("Isla Phillips")
zara = Person("Zara Tindall")
mia = Person("Mia Tindall")
queen.add_child(charles)
queen.add_child(anne)
queen.add_child(andrew)
queen.add_child(edward)
charles.add_child(william)
charles.add_child(henry)
william.add_child(george)
william.add_child(charlotte)
andrew.add_child(beatrice)
andrew.add_child(eugenie)
edward.add_child(louise)
edward.add_child(james)
anne.add_child(peter)
anne.add_child(zara)
peter.add_child(savannah)
peter.add_child(isla)
zara.add_child(mia)


# zkuste si:

# print(bart.name)
# print(bart.parent.name)
# print(siblings(bart, lisa))
# print(siblings(homer, bart))
# print(siblings(maggie, abe))
# print(count_offspring(abe))

# p = oldest_ancestor(charlotte)
# print(p.name)

# draw_family_tree(abe)
# draw_family_tree(queen)
# draw_family_tree(charles)

# create_family_tree_graph(queen)
